import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_svg/svg.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/routines/bloc/functions_bloc/functions_bloc_bloc.dart';
import 'package:syncrow_web/pages/routines/bloc/routine_bloc/routine_bloc.dart';
import 'package:syncrow_web/pages/routines/models/device_functions.dart';
import 'package:syncrow_web/pages/routines/models/gateway.dart';
import 'package:syncrow_web/pages/routines/widgets/dialog_footer.dart';
import 'package:syncrow_web/pages/routines/widgets/dialog_header.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class GatewayIfDialog extends StatefulWidget {
  const GatewayIfDialog({
    required this.uniqueCustomId,
    required this.functions,
    required this.deviceSelectedFunctions,
    super.key,
  });

  final String? uniqueCustomId;
  final List<DeviceFunction> functions;
  final List<DeviceFunctionData> deviceSelectedFunctions;

  @override
  State<GatewayIfDialog> createState() => _GatewayIfDialogState();
}

class _GatewayIfDialogState extends State<GatewayIfDialog> {
  late final List<GatewayFunctions> _gatewayFunctions;

  @override
  void initState() {
    super.initState();
    _gatewayFunctions = widget.functions.whereType<GatewayFunctions>().toList();
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      contentPadding: EdgeInsets.zero,
      content: BlocBuilder<FunctionBloc, FunctionBlocState>(
        builder: (context, state) {
          final selectedFunction = state.selectedFunction;
          return Container(
            width: selectedFunction != null ? 600 : 360,
            height: 450,
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.circular(20),
            ),
            padding: const EdgeInsets.only(top: 20),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                const DialogHeader('Gateway Conditions'),
                Expanded(child: _buildMainContent(context, state)),
                _buildDialogFooter(context, state),
              ],
            ),
          );
        },
      ),
    );
  }

  Widget _buildDialogFooter(BuildContext context, FunctionBlocState state) {
    return DialogFooter(
      onCancel: () => Navigator.pop(context),
      onConfirm: state.addedFunctions.isNotEmpty
          ? () {
              context.read<RoutineBloc>().add(
                    AddFunctionToRoutine(
                      state.addedFunctions,
                      widget.uniqueCustomId ?? '-1',
                    ),
                  );
              Navigator.pop(
                context,
                {'deviceId': widget.functions.firstOrNull?.deviceId},
              );
            }
          : null,
      isConfirmEnabled: state.selectedFunction != null,
    );
  }

  Widget _buildMainContent(BuildContext context, FunctionBlocState state) {
    final selectedFunction = state.selectedFunction;
    final selectedOperationName = state.selectedOperationName;
    final selectedFunctionData = state.addedFunctions.firstWhere(
      (f) => f.functionCode == selectedFunction,
      orElse: () => DeviceFunctionData(
        entityId: '',
        functionCode: selectedFunction ?? '',
        operationName: '',
        value: null,
      ),
    );

    return Row(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        _buildFunctionList(context),
        if (state.selectedFunction != null)
          Expanded(
            child: _buildValueSelector(
              context: context,
              selectedFunction: selectedFunction ?? '',
              selectedFunctionData: selectedFunctionData,
              acFunctions: _gatewayFunctions,
              operationName: selectedOperationName ?? '',
            ),
          ),
      ],
    );
  }

  Widget _buildFunctionList(BuildContext context) {
    return SizedBox(
      width: 360,
      child: ListView.separated(
        shrinkWrap: false,
        itemCount: _gatewayFunctions.length,
        separatorBuilder: (context, index) => const Padding(
          padding: EdgeInsets.symmetric(horizontal: 40.0),
          child: Divider(color: ColorsManager.dividerColor),
        ),
        itemBuilder: (context, index) {
          final function = _gatewayFunctions[index];
          return ListTile(
            leading: SvgPicture.asset(
              function.icon,
              width: 24,
              height: 24,
              placeholderBuilder: (context) => const SizedBox(
                width: 24,
                height: 24,
              ),
            ),
            title: Text(
              function.operationName,
              style: context.textTheme.bodyMedium,
            ),
            trailing: const Icon(
              Icons.arrow_forward_ios,
              size: 16,
              color: ColorsManager.textGray,
            ),
            onTap: () => context.read<FunctionBloc>().add(
                  SelectFunction(
                    functionCode: function.code,
                    operationName: function.operationName,
                  ),
                ),
          );
        },
      ),
    );
  }

  static Widget _buildValueSelector({
    required BuildContext context,
    required String selectedFunction,
    required DeviceFunctionData? selectedFunctionData,
    required List<GatewayFunctions> acFunctions,
    AllDevicesModel? device,
    required String operationName,
  }) {
    final selectedGatewayFunctions = acFunctions.firstWhere(
      (f) => f.code == selectedFunction,
    );
    final values = selectedGatewayFunctions.getOperationalValues();

    return _buildOperationalValuesList(
      context: context,
      values: values,
      selectedValue: selectedFunctionData?.value,
      device: device,
      operationName: operationName,
      selectCode: selectedFunction,
      selectedFunctionData: selectedFunctionData,
    );
  }

  static Widget _buildOperationalValuesList({
    required BuildContext context,
    required List<GatewayOperationalValue> values,
    required dynamic selectedValue,
    AllDevicesModel? device,
    required String operationName,
    required String selectCode,
    DeviceFunctionData? selectedFunctionData,
  }) {
    return ListView.builder(
      itemCount: values.length,
      itemBuilder: (context, index) {
        final value = values[index];
        final isSelected = selectedValue == value.value;
        return ListTile(
          leading: SvgPicture.asset(
            value.icon,
            width: 24,
            height: 24,
            placeholderBuilder: (context) => Container(
              width: 24,
              height: 24,
              color: Colors.transparent,
            ),
          ),
          title: Text(
            value.description,
            style: context.textTheme.bodyMedium,
          ),
          trailing: Icon(
            isSelected ? Icons.radio_button_checked : Icons.radio_button_unchecked,
            size: 24,
            color: isSelected
                ? ColorsManager.primaryColorWithOpacity
                : ColorsManager.textGray,
          ),
          onTap: () {
            if (!isSelected) {
              context.read<FunctionBloc>().add(
                    AddFunction(
                      functionData: DeviceFunctionData(
                        entityId: device?.uuid ?? '',
                        functionCode: selectCode,
                        operationName: operationName,
                        value: value.value,
                        condition: selectedFunctionData?.condition,
                        valueDescription: selectedFunctionData?.valueDescription,
                      ),
                    ),
                  );
            }
          },
        );
      },
    );
  }
}
